#include "mpi.h"

#define MAX_COUNT 1000

int main( argc, argv )
int argc;
char **argv;
{
  MPI_File   fh;
  MPI_Info   info;
  MPI_Status status;
  char       *filename;
  int        buffer[MAX_COUNT];
  int        count, rank;

  MPI_Init( &argc, &argv );
  
  MPI_Info_create( &info );
  /* MPI_INFO_NULL may be used instead of this info */
  MPI_Info_set( info, "collective_buffering", "true" );

  /* The filename is implementation-specific (just as it is
     for all environments).  This assumes that a simple file name will 
     work.  The file name may be over-ridden from the command line */
  filename = "sample.input";
  if (argc > 1 && argv[1]) filename = argv[1];

  /* All processes open the file.  Complex headers (with fixed-sized
     fields) can be read using an MPI datatype */
  MPI_File_open( MPI_COMM_WORLD, filename, 
		 MPI_MODE_RDONLY | MPI_MODE_UNIQUE_OPEN,
		 info, &fh );

  MPI_File_read_all( fh, buffer, MAX_COUNT, MPI_INT, &status );

  /* All processes now have all of the data */

  /* Find out how much data was read */
  MPI_Get_count( &status, MPI_INT, &count );

  /* Check that we all  got the correct data */
  for (i=0; i<count; i++) {
    if (buffer[i] != i) 
      fprintf( stderr, "Expected %d but read %d\n", i, buffer[i] );
  }

  MPI_Comm_rank( MPI_COMM_WORLD, &rank );
  if (rank == 0) {
    printf( "Read %d integers\n", count );
  }
  MPI_File_close( &fh );
  MPI_Info_free( &info );

  MPI_Finalize( );
  return 0;
}
